var slashy = new function() {
	// public functions
	// ****************

	this.init = function() {
		slashy.pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.slashy.");

		// developer or not?
		slashy.dev = false;
		try {
			slashy.dev = slashy.pref.getBoolPref("developer");
		} catch(err) {
		}

		if (slashy.dev && typeof(slashy.started) == "undefined") {
			traceLog("Slashy - 1.6.0");
			slashy.started = 1;
		}

		// add pref observer so the toolbar button updates when user changes prefs via about:config
    var pbi = slashy.pref.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver("", this, false);

		// show the first time welcome note
		if (typeof(slashy.firsttime) == "undefined" && typeof(BrowserCustomizeToolbar) == "function") {
			slashy.firsttime = false;
			if (slashy.pref.getBoolPref("firsttime")) {
				if (!document.getElementById("slashy-toolbar")) {
					traceLog("Try to show the firsttime note");
					window.setTimeout("slashy.showfirsttime()", 50);
				}
			} else {
				slashy.pref.setBoolPref("firsttime", false);
			}
		}

		// is it enabled or disabled?
		if (typeof(slashy.enabled) == "undefined") {
			try {
				slashy.enabled = slashy.pref.getBoolPref("enabled");
			} catch(err) {
				slashy.enabled = true;
			}
		}
		changeButton(slashy.enabled);
		if (!slashy.enabled) {
			return;
		}

		// cache the URL Components
		if (typeof(this.URL) == "undefined") {
			traceLog("Create URL object");
			try {
				this.URL = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURL);
			} catch(err) {
			}
		}

		// do the fixes
		slashy.fixSlash();
	},

	this.observe = function(aSubject, aTopic, aData) {
		if (aTopic != "nsPref:changed")
			return;
		switch (aData) {
			case "enabled":
				slashy.enabled = slashy.pref.getBoolPref("enabled");
				changeButton(slashy.enabled);
				break;
			case "developer":
				slashy.dev = slashy.pref.getBoolPref("developer");
				break;
		}
	},

	this.showfirsttime = function() {
		var bs = document.getElementById("bundle_slashy");
		var message = bs.getString("slashy-firsttime-message");
		var checkbox = bs.getString("slashy-firsttime-checkbox");
		var showAgain = {value: false};
		var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
		var rv = ps.confirmEx(window, "Slashy", message, (Components.interfaces.nsIPromptService.BUTTON_TITLE_YES * Components.interfaces.nsIPromptService.BUTTON_POS_0) + (Components.interfaces.nsIPromptService.BUTTON_TITLE_NO * Components.interfaces.nsIPromptService.BUTTON_POS_1), null, null, null, checkbox, showAgain);
		if (rv == 0) {
			BrowserCustomizeToolbar();
		}
		slashy.pref.setBoolPref("firsttime", showAgain.value);
	},

	this.objFix = function(aObj) {
		var changed = false;

		// the reg exp to look for and replace
		var re = new RegExp("(\/%5C|\\\\|%5C)", "g");
		// dont fix these link types
		var relink = new RegExp("^(javascript|file|http://localhost|http://127.0.0.1)", "i");
	
		if (!relink.test(aObj)) {
			if (re.test(aObj)) {
				this.URL.spec = aObj;
				re = new RegExp("(\/%5C|\\\\|%5C)", "g");
				if (re.test(this.URL.filePath)) {
					traceLog("Bad URL <" + aObj + ">");
					changed = aObj.replace(re, "\/");
					traceLog("Fixed <" + changed + ">");
				} else {
					traceLog("Slash not found in filePath <" + this.URL.filePath + "> part of <" + aObj + ">");
				}
			} else {
				// traceLog("No fix needed <" + aObj + ">");
			}
		} else {
			// traceLog("Skipped <" + aObj + ">");
		}
		return changed;
	},

	this.fixDoc = function(aDoc) {
	  var changed;
	  var num = 0;

		// are there any links?
		if ("links" in aDoc && aDoc.links.length > 0) {
			var lnks = aDoc.links;
			for (var i = 0; i < lnks.length; ++i) {
				if (lnks[i].href) {
					changed = slashy.objFix(lnks[i].href)
					if (changed) {
						lnks[i].href = changed
						num++;
					}
				}
			}
		}
	
		// are there any images?
		if ("images" in aDoc && aDoc.images.length > 0) {
			var imgs = aDoc.images;
			for (var i = 0; i < imgs.length; ++i) {
				if (imgs[i].src) {
					changed = slashy.objFix(imgs[i].src)
					if (changed) {
						imgs[i].src = changed
						num++;
					}
				}
			}
		}

		if (num) {
			var stat = document.getElementById("statusbar-display") || document.getElementById("statusText");
			if (stat) {
				var re = new RegExp("#1", "g");
				stat.label = document.getElementById("bundle_slashy").getString("slashy-changed").replace(re, num);
			}
			traceLog("Number of fixed links/images: " + num);
		}
	},

	this.fixSlash = function() {
		// try to get the current document
		var doc;
	
		try {
			doc = window.top.content.document;
		} catch(err) {
			traceLog("Unable to get doc due to: " + err);
		}
		if (doc) {
			slashy.doDoc(doc);
		}
	},

	this.doDoc = function(aDoc) {
		// fix document
		slashy.fixDoc(aDoc);

		// fix frames
		var frames = aDoc.getElementsByTagName("frame");
		if (typeof(frames.length) != "undefined" && frames.length) {
			for (var i = 0; i < frames.length; i++) {
				var fdoc;
				try {
					fdoc = frames[i].contentDocument;
				} catch(err) {
					traceLog("Unable to get fdoc due to: " + err);
				}
				if (fdoc) {
					slashy.doDoc(fdoc);
				}
			}
		}
	},

	this.doButton = function() {
		if (typeof(slashy.enabled) == "undefined") {
			slashy.enabled = false;
		} else {
			slashy.enabled = !slashy.enabled;
		}
		slashy.pref.setBoolPref("enabled", slashy.enabled);
		changeButton(slashy.enabled);
	}

	// private functions
	// *****************

	function changeButton(aBool) {
		var but = document.getElementById("slashy-toolbar");
		if (but) {
			but.setAttribute("status", aBool);
		}
	};

	function traceLog(aText, aForce) {
		if (aForce || slashy.dev) {
			try {
				var CONSOLE_SERVICE = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
				CONSOLE_SERVICE.logStringMessage("slashy: " + aText);
			} catch(err) {
			}
		}
	};
}

// start it every time a page has loaded
window.addEventListener("load", function(e) {slashy.init(e);}, true);